using System;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace Mycos.Glib
{
	#region Wll Side Enumeration.
	/// <summary>
	/// Wall Side
	/// </summary>
	public enum WallSide
	{

		/// <summary>
		/// Left
		/// </summary>
		Left,

		/// <summary>
		/// Right
		/// </summary>
		Right,

		/// <summary>
		/// Top
		/// </summary>
		Top,

		/// <summary>
		/// Bottom
		/// </summary>
		Bottom,

		/// <summary>
		/// Back
		/// </summary>
		Back,

		/// <summary>
		/// Front
		/// </summary>
		Front
	}
	#endregion

	/// <summary>
	/// Summary description for DrawingUtils.
	/// </summary>
	public class DrawingUtils
	{
		/// <summary>
		/// Initializes a new instance of the DrawingUtils class.
		/// </summary>
		public DrawingUtils()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		#region Static Method.

		/// <summary>
		/// Gets the path of the round corner rectangle.
		/// </summary>
		/// <param name="bound">The RectangleF struct that want to source of the round corner.</param>
		/// <param name="radius">Corner Radius.</param>
		/// <returns>Graphics path of the round corner.</returns>
		public static GraphicsPath GetRoundRectangle(RectangleF bound, float radius)
		{
			GraphicsPath gp = new GraphicsPath();

			float x = bound.X;
			float y = bound.Y;
			float width = bound.Width;
			float height = bound.Height;
			            
			// Top
			gp.AddLine(x + radius, y, x + width - radius, y);
			gp.AddArc(x + width - radius, y, radius, radius, 270, 90);

			// Right
			gp.AddLine(x + width, y + radius, x + width, y + height - radius);
			gp.AddArc(x + width - radius, y + height - radius, radius, radius, 0, 90);

			// Bottom
			gp.AddLine(x + width - radius, y + height, x + radius, y + height);
			gp.AddArc(x, y + height - radius, radius, radius, 90, 90);

			// Left
			gp.AddLine(x, y + height - radius, x, y + radius);
			gp.AddArc(x, y, radius, radius, 180, 90);


			gp.CloseFigure();

			return gp;
		}//end method 


		/// </summary>
		/// <param name="g">Graphics Object</param>
		/// <param name="front">Rectangle</param>
		/// <param name="depth">Depth of the cylinder.</param>
		/// <param name="fillColor">Fill Color</param>
		/// <param name="borderColor">Border Color</param>
		/// <param name="outLine">Indicated Outline</param>
		public static void FillCylinder(Graphics g, RectangleF front, SizeF depth, Color fillColor, Color borderColor, bool outLine)
		{
			if ( front.Width <= 0 || front.Height <= 0 )
				return;

			// Make Back Side Area
			RectangleF aback = front;

			// Make Depth
			aback.X += depth.Width;
			aback.Y -= depth.Height;

			// Create Top and Bottom Plane.
			RectangleF bottomPlane;
			RectangleF topPlane;

			// Create Graphics Object
			GraphicsPath gp = new GraphicsPath();
				
			topPlane = new RectangleF((front.X+aback.X)/2, aback.Location.Y, front.Width, front.Y - aback.Y);

			bottomPlane = new RectangleF(new PointF((front.X+aback.X)/2, aback.Bottom), new SizeF(front.Width, front.Y - aback.Y ));

			// Brush
			SolidBrush brush = new SolidBrush(fillColor);

			// Border Pen
			Pen borderPen = new Pen(borderColor);

			/***************
			 *   Bottom    *
			 * ************/
			// Make GP On Bottom
			gp.AddEllipse(bottomPlane);

			// Get Bottom color
			brush.Color = GetSideColor(fillColor, WallSide.Bottom);

			// Fill Bottom Plane
			g.FillPath(brush, gp);

			// Check Draw Border
			if ( outLine )
				g.DrawPath(borderPen, gp);

			gp.Reset();
			gp.AddArc(topPlane, 0, 180);
			gp.AddLine(gp.GetLastPoint(), new PointF((front.X+aback.X)/2, (front.Bottom+aback.Bottom)/2.0f));
			gp.AddArc(bottomPlane, 180, -180);
			gp.CloseFigure();

			/***************
			 *     Body    *
			 * ************/
			// Color For Body is real Fill Color.
			brush.Color = fillColor;

			// Fill Body
			g.FillPath(brush, gp);

			// Shadow of the Body
			FillCylinderShadow(g, front, gp, false);

			// Check Draw Border
			if ( outLine )
				g.DrawPath(borderPen, gp);

			/***************
			 *     Top     *
			 * ************/
			gp.Reset();
			gp.AddEllipse(topPlane);

			// Get Bottom color
			brush.Color = GetSideColor(fillColor, WallSide.Top);

			// Fill Top Plane
			g.FillPath(brush, gp);

			// Check Draw Border
			if ( outLine )
				g.DrawPath(borderPen, gp);
			
			// Dispose
			gp.Dispose();
			brush.Dispose();
			borderPen.Dispose();
		}//End Method Fill

		/// <summary>
		/// Draw the Shadow for Cylinder.
		/// </summary>
		/// <param name="g">Graphics Object</param>
		/// <param name="bound">Front Bound</param>
		/// <param name="gp">Graphics Path</param>
		/// <param name="horizontal">Indicated use draw Horionzotal</param>
		public static void FillCylinderShadow(Graphics g, RectangleF bound, GraphicsPath gp, bool horizontal)
		{
			float angle = -180.0f;

			if ( horizontal )
				angle = -90.0f;

			LinearGradientBrush linGrBrush = new LinearGradientBrush(
				bound,
				Color.FromArgb(100, 128, 128, 128),     // Opaque black 
				Color.FromArgb(100, Color.White),
				angle);  // Opaque red

			linGrBrush.SetSigmaBellShape(0.70f);

			g.FillPath(linGrBrush, gp);

			linGrBrush.Dispose();
		}//end Method FillCylinderShadow

		/// <summary>
		/// Get Side Color
		/// </summary>
		/// <param name="color"></param>
		/// <param name="side"></param>
		/// <returns>Color</returns>
		public static Color GetSideColor(Color color, WallSide side)
		{
			float bright = 1.0f;

			byte a = color.A;
			byte r = color.R;
			byte g = color.G;
			byte b = color.B;

			switch(side)
			{
				case WallSide.Back :
				{
					bright = 1.0f;
					break;
				}

				case WallSide.Front :
				{
					bright = 1.0f;
					break;
				}
				
				case WallSide.Left :
				{
					bright = 0.7f;
					break;
				}

				case WallSide.Right :
				{
					bright = 0.5f;
					break;
				}

				case WallSide.Top :
				{
					bright = 0.8f;
					break;
				}

				case WallSide.Bottom  :
				{
					bright = 0.4f;
					break;
				}
	
			}//end swithc case

			r = (byte)(r/2 + r*bright/2);
			g = (byte)(g/2 + g*bright/2);
			b = (byte)(b/2 + b*bright/2);

			return Color.FromArgb(a, r, g, b);
		}
		#endregion
	}
}
