using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Windows.Forms;

namespace Mycos.Controls.WinControls
{
	/// <summary>
	/// Internal mouse state.
	/// </summary>
	enum MouseState
	{
		Normal,

		HOver,

		Hot
	}

	/// <summary>
	/// Summary description for RoundControl.
	/// </summary>
	public class RoundControl : System.Windows.Forms.UserControl
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		#region Fields.

		private GraphicsPath		controlPath;
		private float				cornerRadius;
		private float				borderWidth;
		private Color				borderColor;				

		#endregion

		#region Event.

		/// <summary>
		/// Occurs when the Corner Radius was changed.
		/// </summary>
		public event EventHandler CornerRadiusChanged;

		/// <summary>
		/// Occurs when the Border Width was changed.
		/// </summary>
		public event EventHandler BorderWidthChanged;
		#endregion

		/// <summary>
		/// Initializes a new instance of the CloseButton class.
		/// </summary>
		public RoundControl()
		{
			//
			// Initialize Value
			//
			this.cornerRadius = 15;
			this.borderColor = Color.White;
			this.borderWidth = 0f;

			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			SetStyle(ControlStyles.UserPaint, true); 
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.DoubleBuffer, true);  
			SetStyle(ControlStyles.ResizeRedraw, true);
			SetStyle(ControlStyles.SupportsTransparentBackColor, true);

			
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// CloseButton
			// 
			this.Name = "CloseButton";
			this.Size = new System.Drawing.Size(18, 18);

		}
		#endregion

		#region Public Properties.

		[Category("Round Styles"),
		DefaultValue(15f)]
		public float CornerRadius
		{
			get
			{
				return this.cornerRadius;
			}
			set
			{
				if ( this.cornerRadius != value )
				{
					this.cornerRadius = value;
					this.OnCornerRadiusChanged(EventArgs.Empty);
				}
			}
		}

		[Category("Round Styles"),
		DefaultValue(typeof(Color), "White")]
		public Color BorderColor
		{
			get
			{
				return this.borderColor;
			}
			set
			{
				if ( this.borderColor != value )
				{
					this.borderColor = value;
					this.Invalidate();
				}
			}
		}

		[Category("Round Styles")]
		[DefaultValue("0f")]
		public float BorderWidth
		{
			get
			{
				return this.borderWidth;
			}
			set
			{
				if ( this.borderWidth != value )
				{
					this.borderWidth = value;
					this.OnBorderWidthChanged(EventArgs.Empty);
				}
			}
		}
		#endregion

		#region Protected Properties.

		/// <summary>
		/// Gets the Control Path.
		/// </summary>
		protected virtual GraphicsPath ControlPath
		{
			get
			{
				if ( this.controlPath == null )
					this.controlPath = DefineControlPath();

				return this.controlPath;
			}
		}

		#endregion

		#region Layout Code.

		/// <summary>
		/// This Method much be return the control Path.
		/// </summary>
		/// <returns>The new Control path.</returns>
		protected virtual GraphicsPath DefineControlPath()
		{
			return Mycos.Glib.DrawingUtils.GetRoundRectangle(this.ClientRectangle, this.cornerRadius);
		}

		/// <summary>
		/// Overidden OnResize to Control the Control Path.
		/// </summary>
		/// <param name="e"></param>
		protected override void OnResize(EventArgs e)
		{
			base.OnResize(e);

			this.controlPath = null;
			this.Region = new Region(ControlPath);
		}
		#endregion

		#region Paint Code.

		/// <summary>
		/// Overridden OnPaint for custom painting.
		/// </summary>
		/// <param name="e">A System.Windows.Forms.PaintEventArgs that contains the event data.</param>
		protected override void OnPaint(PaintEventArgs e)
		{
			base.OnPaint(e);
			Graphics g = e.Graphics;
			g.SmoothingMode = SmoothingMode.AntiAlias;

			if ( this.borderWidth > 0 && this.ControlPath != null)
			{
				Pen borderPen = new Pen(this.BorderColor, this.BorderWidth);
				borderPen.Alignment = PenAlignment.Center;

				GraphicsPath innerPath = (GraphicsPath)this.ControlPath.Clone();

				float halfBorder = BorderWidth/2.0f;
				Matrix m = new Matrix();
				m.Scale(1F-(BorderWidth/this.ClientRectangle.Width), 1F-(BorderWidth/this.ClientRectangle.Height));
				m.Translate(halfBorder, halfBorder);
				innerPath.Transform(m);

				g.DrawPath(borderPen, innerPath);

				// Dispose
				borderPen.Dispose();
				innerPath.Dispose();
				m.Dispose();
			}//end if borderWidth
		}//end Paint
		#endregion

		#region EventHandler.
		/// <summary>
		/// Raise when the CornerRadius value was changed.
		/// </summary>
		/// <param name="e">EventArgs argument.</param>
		protected virtual void OnCornerRadiusChanged(EventArgs e)
		{
			if ( this.CornerRadiusChanged != null )
			{
				this.controlPath = null;
				this.CornerRadiusChanged(this, e);
			}

			this.Invalidate();
		}

		/// <summary>
		/// Raise when the Border width value was changed.
		/// </summary>
		/// <param name="e">EventArgs argument.</param>
		protected virtual void OnBorderWidthChanged(EventArgs e)
		{
			if ( this.BorderWidthChanged != null )
			{
				this.BorderWidthChanged(this, e);
			}

			this.Invalidate();
		}
		#endregion
	}
}
